/* SystemTap probe support for GDB.

   Copyright (C) 2011 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#include "defs.h"
#include "stap-probe.h"
#include "vec.h"
#include "ui-out.h"
#include "gdb_regex.h"
#include "objfiles.h"
#include "arch-utils.h"
#include "command.h"
#include "filenames.h"
#include "value.h"
#include "exceptions.h"
#include "ax.h"
#include "ax-gdb.h"
#include "user-regs.h"
#include "complaints.h"
#include "cli/cli-utils.h"
#include "linespec.h"

#include <ctype.h>

/* This define is used to inform whether we are compiling an agent
   expression or not.  */

#define STAP_COMPILING_AGENT_EXPR_P(eval_info) \
  (eval_info->aexpr != NULL)

/* The various possibilities of bitness defined for a probe's argument.

   The relationship is:

   - STAP_ARG_BITNESS_UNDEFINED:  The user hasn't specified the bitness.
   - STAP_ARG_BITNESS_32BIT_UNSIGNED:  argument string starts with `4@'.
   - STAP_ARG_BITNESS_32BIT_SIGNED:  argument string starts with `-4@'.
   - STAP_ARG_BITNESS_64BIT_UNSIGNED:  argument string starts with `8@'.
   - STAP_ARG_BITNESS_64BIT_SIGNED:  argument string starts with `-8@'.  */

enum stap_arg_bitness
{
  STAP_ARG_BITNESS_UNDEFINED,
  STAP_ARG_BITNESS_32BIT_UNSIGNED,
  STAP_ARG_BITNESS_32BIT_SIGNED,
  STAP_ARG_BITNESS_64BIT_UNSIGNED,
  STAP_ARG_BITNESS_64BIT_SIGNED,
};

/* The following structure represents a single argument for the probe.  */

struct stap_probe_arg
{
  /* The bitness of this argument.  */
  enum stap_arg_bitness bitness;

  /* The string representing this argument.  */
  char *arg_str;
};

/* The maximum number of arguments that a probe can have,
   as defined in <sys/sdt.h>.  */

#define STAP_MAX_ARGS 10

/* Structure that holds information about all arguments of a probe.  */

struct stap_args_info
{
  /* The number of valid parsed arguments.  */
  int n_args;

  /* The probe to which these arguments belong.  */
  struct stap_probe *probe;

  /* Information about each argument.  */
  struct stap_probe_arg *arg;
};

/* Structure that contains all the necessary information to evaluate
   an expression.  */

struct stap_evaluation_info
{
  /* The constant pointer which holds the expression. This is primarily
     used for printing error messages.  Evaluation functions should
     not modify this pointer directly; instead, they should use the
     EXP_BUFFER pointer below.  */
  const char *saved_expr;

  /* Modifiable version of the above pointer.  */
  char *exp_buf;

  /* The pointer to the current gdbarch.  */
  struct gdbarch *gdbarch;

  /* The pointer to the current frame, used when accessing registers'
     contents.  */
  struct frame_info *frame;

  /* The bitness specified for this argument.  */
  enum stap_arg_bitness bitness;

  /* If the above flag is true (one), this field will contain the
     pointer to the agent expression.  */
  struct agent_expr *aexpr;

  /* The value we are modifying (for agent expression).  */
  struct axs_value *avalue;
};

/* This dummy variable is used when parsing a probe's argument fails.
   In this case, the number of arguments for this probe is zero, so that's
   why this variable is useful.  */

static struct stap_args_info dummy_stap_args_info =
  { 0, NULL, NULL };

static struct value *stap_evaluate_probe_argument_2
  (struct stap_evaluation_info *eval_info,
   struct value *lhs, int prec);

static struct value *stap_evaluate_conditionally
  (struct stap_evaluation_info *eval_info);

/* Helper function which decides to skip whitespaces or not in a probe's
   argument string.  Basically, if we are inside a parenthesis expression
   (i.e., inside a subexpression), we can skip whitespaces; otherwise we
   cannot.  */

static void
stap_skip_whitespace_cond (char **s, int inside_paren)
{
  if (inside_paren)
    *s = skip_spaces (*s);
}

/* Helper function which parses a single argument in a probe's argument
   string, based on various rules (which can be learned from the `gas'
   manual).  It returns 1 on success, or 0 otherwise.  */

static int
stap_parse_arg (const char **p)
{
  char *cur = (char *) *p;
  int done = 0;
  int paren_open = 0;

  while (!done)
    {
      switch (*cur)
	{
	case ' ': case 0:
	  /* If we're here, then we have already parsed everything
	     from this argument.  */
	  if (paren_open)
	    return 0;
	  done = 1;
	  break;

	case '(':
	  ++paren_open;
	  ++cur;
	  stap_skip_whitespace_cond (&cur, paren_open);
	  break;

	case ')':
	  if (!paren_open)
	    return 0;

	  --paren_open;
	  ++cur;
	  if (paren_open)
	    cur = skip_spaces (cur);
	  break;

	case '+': case '-':
	case '*': case '/':
	case '>': case '<': case '|': case '&':
	case '^': case '!':
	    {
	      char c = *cur;

	      ++cur;
	      switch (*cur)
		{
		case '>':
		  if (c != '<' && c != '>')
		    return 0;

		  ++cur;
		  break;

		case '<':
		  if (c != '<')
		    return 0;

		  ++cur;
		  break;

		case '=':
		  if (c != '=' && c != '<' && c != '>' && c != '!')
		    return 0;

		  ++cur;
		  break;

		case '|':
		  if (c != '|')
		    return 0;

		  ++cur;
		  break;

		case '&':
		  if (c != '&')
		    return 0;

		  ++cur;
		  break;

		default:
		  break;
		}
	      /* Infix operators take two arguments, one on either
		 side.  Skipping the whitespaces that may happen on the
		 right side.  */
	      stap_skip_whitespace_cond (&cur, paren_open);
	    }
	  break;

	case '%':
	    {
	      ++cur;
	      stap_skip_whitespace_cond (&cur, paren_open);
	      if (*cur >= 'a' && *cur <= 'z')
		{
		  /* We're dealing with a register name.  */
		  while (isalnum (*cur))
		    ++cur;

		  stap_skip_whitespace_cond (&cur, paren_open);

		  /* Some registers (e.g. floating-point register stack
		     registers on Intel i386) have the following syntax:

		     `%st(0)', `%st(1)', and so on.

		     So it's ok to expect parenthesis here.  */
		  if (*cur == '(')
		    {
		      ++cur;
		      stap_skip_whitespace_cond (&cur, paren_open);
		      if (!isdigit (*cur))
			/* This is an error, since we only expect numbers
			   inside this parenthesis.  */
			return 0;
		      ++cur;
		      stap_skip_whitespace_cond (&cur, paren_open);
		      if (*cur != ')')
			/* We only expect one number.  */
			return 0;
		      ++cur;
		      stap_skip_whitespace_cond (&cur, paren_open);
		    }
		}
	    }
	  break;

	case '$':
	    {
	      /* This is an integer constant.  */
	      ++cur;
	      stap_skip_whitespace_cond (&cur, paren_open);

	      while (isdigit (*cur))
		++cur;

	      stap_skip_whitespace_cond (&cur, paren_open);
	    }
	  break;

	case '0': case '1': case '2': case '3': case '4':
	case '5': case '6': case '7': case '8': case '9':
	    {
	      char *old = cur;

	      /* Number.  */
	      while (isdigit (*cur))
		++cur;

	      /* We have to do a lookahead here, because the user may
		 input `2 + 2' (with spaces), and this is not an error.  */
	      cur = skip_spaces (cur);

	      switch (*cur)
		{
		case '+': case '-':
		  /* We may find the `@' sign, and it means that the
		     argument has finished, so we shouldn't advance the
		     pointer.  */
		  if (cur[1] && (cur[1] == '4' || cur[1] == '8')
		      && cur[2] && cur[2] == '@')
		    {
		      cur = old;
		      goto fin;
		    }
		  break;

		case '*': case '/': case '>': case '<':
		case '|': case '&': case '^': case '!':
		  /* This is a binary operation, which means we'll
		     have to find another number after the operator.  */
		  break;

		case '(':
		  /* We may also have sentences in the form:

		     `4 (%rax)'  */
		  break;
		}
	    }
fin:
	  break;
	}
    }

  *p = cur;

  return 1;
}

/* Helper function which is responsible for freeing the space allocated to
   hold information about a probe's arguments.  */

static void
stap_free_args_info (void *args_info_ptr)
{
  struct stap_args_info *a = (struct stap_args_info *) args_info_ptr;
  int i;

  for (i = 0; i < a->n_args; i++)
    {
      xfree (a->arg[i].arg_str);
    }

  xfree (a->arg);
  xfree (a);
}

/* Function which parses an argument string from PROBE, correctly splitting
   the arguments and storing their information in properly ways.  This function
   only separates the arguments, but does not evaluate them.

   Consider the following argument string:

   `4@%eax 4@$10'

   We have two arguments, `%eax' and `$10', both with 32-bit unsigned bitness.
   This function basically handles them, properly filling some structures with
   this information.  */

static void
stap_parse_probe_arguments (struct stap_probe *probe)
{
  struct stap_args_info *args_info;
  struct cleanup *back_to;
  const char *cur = probe->args;
  int current_arg = -1;
  /* This is a state-machine parser, which means we will always be
     in a known state when parsing an argument.  The state could be
     either `NEW_ARG' if we are parsing a new argument, `BITNESS' if
     we are parsing the bitness-definition part (i.e., `4@'), or
     `PARSE_ARG' if we are actually parsing the argument part.  */
  enum
    {
      NEW_ARG,
      BITNESS,
      PARSE_ARG,
    } current_state;

  /* For now, we assume everything is not going to work.  */
  probe->parsed_args = &dummy_stap_args_info;

  if (!cur || !*cur || *cur == ':')
    return;

  args_info = xmalloc (sizeof (struct stap_args_info));
  back_to = make_cleanup (stap_free_args_info, args_info);
  args_info->arg = xcalloc (STAP_MAX_ARGS, sizeof (struct stap_probe_arg));

  /* Ok, let's start.  */
  current_state = NEW_ARG;

  while (*cur)
    {
      switch (current_state)
	{
	case NEW_ARG:
	  ++current_arg;

	  if (current_arg >= STAP_MAX_ARGS)
	    {
	      complaint (&symfile_complaints,
			 _("probe `%s' has more arguments than the maximum "
			   "allowed"), probe->name);
	      do_cleanups (back_to);
	      return;
	    }

	  current_state = BITNESS;
	  break;

	case BITNESS:
	    {
	      enum stap_arg_bitness b;
	      int got_minus = 0;

	      /* We expect to find something like:

		 N@OP

		 Where `N' can be [+,-][4,8].  This is not mandatory, so
		 we check it here.  If we don't find it, go to the next
		 state.  */
	      if ((*cur == '-' && cur[1] && cur[2] != '@')
		  && cur[1] != '@')
		{
		  current_state = PARSE_ARG;
		  args_info->arg[current_arg].bitness
		    = STAP_ARG_BITNESS_UNDEFINED;
		  break;
		}

	      if (*cur == '-')
		{
		  /* Discard the `-'.  */
		  ++cur;
		  got_minus = 1;
		}

	      if (*cur == '4')
		b = got_minus ? STAP_ARG_BITNESS_32BIT_SIGNED
		  : STAP_ARG_BITNESS_32BIT_UNSIGNED;
	      else if (*cur == '8')
		b = got_minus ? STAP_ARG_BITNESS_64BIT_SIGNED
		  : STAP_ARG_BITNESS_64BIT_UNSIGNED;
	      else
		{
		  /* We have an error, because we don't expect anything
		     except 4 and 8.  */
		  complaint (&symfile_complaints,
			     _("unrecognized bitness `%c' for probe `%s'"),
			     *cur, probe->name);
		  do_cleanups (back_to);
		  return;
		}

	      args_info->arg[current_arg].bitness = b;
	      /* Discard the number and the `@' sign.  */
	      cur += 2;
	      /* Move on.  */
	      current_state = PARSE_ARG;
	    }
	  break;

	case PARSE_ARG:
	    {
	      const char *start = cur;

	      if (!stap_parse_arg (&cur))
		{
		  /* We have tried to parse this argument, but it's
		     malformed.  This is an error.  */
		  do_cleanups (back_to);
		  return;
		}

	      args_info->arg[current_arg].arg_str
		= savestring (start, cur - start);
	      /* Start it over again.  */
	      cur = skip_spaces ((char *) cur);
	      current_state = NEW_ARG;
	    }
	  break;
	}

      if (!*cur && current_state != NEW_ARG)
	{
	  /* We reached the end of the argument string, but we're
	     still in the middle of the process of parsing an argument.
	     It means the argument string is malformed.  */
	  complaint (&symfile_complaints,
		     _("malformed argument for probe `%s'"),
		     probe->name);
	  do_cleanups (back_to);
	  return;
	}
    }

  args_info->n_args = current_arg + 1;
  args_info->arg = xrealloc (args_info->arg,
			      args_info->n_args
			      * sizeof (struct stap_probe_arg));
  args_info->probe = probe;

  probe->parsed_args = args_info;

  discard_cleanups (back_to);
}

/* See definition in stap-probe.h.  */

int
stap_get_probe_argument_count (const struct stap_probe *probe)
{
  if (!probe->parsed_args)
    stap_parse_probe_arguments ((struct stap_probe *) probe);

  return probe->parsed_args->n_args;
}

/* Returns the operator precedence level of OP, or zero if the operator
   code was not recognized.
   The levels were taken from the gas manual.  */

static int
stap_get_operator_prec (enum exp_opcode op)
{
  switch (op)
    {
    case BINOP_LOGICAL_OR:
      return 1;

    case BINOP_LOGICAL_AND:
      return 2;

    case BINOP_ADD: case BINOP_SUB:
    case BINOP_EQUAL: case BINOP_NOTEQUAL:
    case BINOP_LESS: case BINOP_LEQ:
    case BINOP_GTR: case BINOP_GEQ:
      return 3;

    case BINOP_BITWISE_IOR: case BINOP_BITWISE_AND:
    case BINOP_BITWISE_XOR: case UNOP_LOGICAL_NOT:
      return 4;

    case BINOP_MUL: case BINOP_DIV: case BINOP_REM:
    case BINOP_LSH: case BINOP_RSH:
      return 5;

    default:
      return 0;
    }
}

/* Given S, this function reads the operator in it and fills the OP
   pointer with its code.  Returns 1 on success, zero if the operator
   was not recognized.  */

static int
stap_get_opcode (char **s, enum exp_opcode *op)
{
  char c = **s;
  int ret = 1;

  *s += 1;

  switch (c)
    {
    case '*':
      *op = BINOP_MUL;
      break;

    case '/':
      *op = BINOP_DIV;
      break;

    case '%':
      {
	if (isalpha (**s))
	  {
	    /* Dealing with a register name.  */
	    ret = 0;
	    break;
	  }

	*op = BINOP_REM;
      }
    break;

    case '<':
      *op = BINOP_LESS;
      if (**s == '<')
	{
	  *s += 1;
	  *op = BINOP_LSH;
	}
      else if (**s == '=')
	{
	  *s += 1;
	  *op = BINOP_LEQ;
	}
      else if (**s == '>')
	{
	  *s += 1;
	  *op = BINOP_NOTEQUAL;
	}
    break;

    case '>':
      *op = BINOP_GTR;
      if (**s == '>')
	{
	  *s += 1;
	  *op = BINOP_RSH;
	}
      else if (**s == '=')
	{
	  *s += 1;
	  *op = BINOP_GEQ;
	}
    break;

    case '|':
      *op = BINOP_BITWISE_IOR;
      if (**s == '|')
	{
	  *s += 1;
	  *op = BINOP_LOGICAL_OR;
	}
    break;

    case '&':
      *op = BINOP_BITWISE_AND;
      if (**s == '&')
	{
	  *s += 1;
	  *op = BINOP_LOGICAL_AND;
	}
    break;

    case '^':
      *op = BINOP_BITWISE_XOR;
      break;

    case '!':
      *op = UNOP_LOGICAL_NOT;
      break;

    case '+':
      *op = BINOP_ADD;
      break;

    case '-':
      *op = BINOP_SUB;
      break;

    case '=':
      if (**s != '=')
	{
	  ret = 0;
	  break;
	}
      *op = BINOP_EQUAL;
      break;

    default:
      /* We didn't find any operator.  */
      *s -= 1;
      return 0;
    }

  return ret;
}

/* Given the operator OPCODE, this function generates agent bytecode
   for it.  */

static void
stap_opcode_to_ax (struct stap_evaluation_info *eval_info,
		   enum exp_opcode opcode)
{
  struct agent_expr *expr = eval_info->aexpr;

  switch (opcode)
    {
    case BINOP_MUL:
      ax_simple (expr, aop_mul);
      break;

    case BINOP_DIV:
      ax_simple (expr, aop_div_signed);
      break;

    case BINOP_REM:
      ax_simple (expr, aop_rem_unsigned);
      break;

    case BINOP_LESS:
      ax_simple (expr, aop_less_signed);
      break;

    case BINOP_LEQ:
      /* A <= B is !(B < A) */
      ax_simple (expr, aop_swap);
      ax_simple (expr, aop_less_signed);
      ax_simple (expr, aop_log_not);
      break;

    case BINOP_GTR:
      /* A > B is B < A */
      ax_simple (expr, aop_swap);
      ax_simple (expr, aop_less_signed);
      break;

    case BINOP_GEQ:
      /* A >= B is !(A < B) */
      ax_simple (expr, aop_less_signed);
      ax_simple (expr, aop_log_not);
      break;

    case BINOP_NOTEQUAL:
      ax_simple (expr, aop_equal);
      ax_simple (expr, aop_log_not);
      break;

    case BINOP_LSH:
      ax_simple (expr, aop_lsh);
      break;

    case BINOP_RSH:
      ax_simple (expr, aop_rsh_unsigned);
      break;

    case BINOP_BITWISE_IOR:
      ax_simple (expr, aop_bit_or);
      break;
     
    case BINOP_LOGICAL_OR:
      error (_("Operator logical-or (`||') not supported yet."));
      break;

    case BINOP_BITWISE_AND:
      ax_simple (expr, aop_bit_and);
      break;

    case BINOP_LOGICAL_AND:
      error (_("Operator logical-and (`&&') not supported yet."));
      break;

    case BINOP_BITWISE_XOR:
      ax_simple (expr, aop_bit_xor);
      break;

    case UNOP_LOGICAL_NOT:
      ax_simple (expr, aop_log_not);
      break;

    case BINOP_ADD:
      ax_simple (expr, aop_add);
      break;

    case BINOP_SUB:
      ax_simple (expr, aop_sub);
      break;

    case BINOP_EQUAL:
      ax_simple (expr, aop_equal);
      break;

    default:
      error (_("Invalid operator."));
    }
}

/* Returns 1 if *S is an operator, zero otherwise.  */

static int
stap_is_operator (char *s)
{
  char op;

  if (!s || !*s)
    return 0;

  op = *s;

  if (*s == '%' && isalpha (s[1]))
    /* Register name.  */
    return 0;

  return (op == '+' || op == '-' || op == '*' || op == '/'
	  || op == '>' || op == '<' || op == '!' || op == '^'
	  || op == '|' || op == '&' || op == '%' || op == '=');
}

/* This function fetches the value of the register whose
   name starts in the expression buffer.  It also applies any register
   displacements (e.g., `-4(%eax)'), and indirects the contents of the
   register (e.g., `(%eax)').  It returns RET if the operation has succeeded,
   or calls `error' otherwise.  */

static struct value *
stap_fetch_reg_value (struct stap_evaluation_info *eval_info,
		      struct value *displacement)
{
  const char *start;
  char *s = eval_info->exp_buf;
  struct gdbarch *gdbarch = eval_info->gdbarch;
  struct frame_info *frame = eval_info->frame;
  enum stap_arg_bitness bitness = eval_info->bitness;
  char *regname;
  int len, regnum, indirect_p = 0;
  struct value *ret = NULL;
  
  /* The function which called us did not check if the expression
     buffer was empty.  */
  gdb_assert (s && *s);

  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
    /* If we are compiling, we cannot return NULL because that would
       lead to errors in future evaluations.  That's why we just make
       this dummy value, representing that the return value of this
       function is not NULL.  */
    ret = value_from_longest (builtin_type (gdbarch)->builtin_int, 0);

  /* Valid register name on x86 platforms are:

     [paren]%{a-z0-9}[paren]

     Let's check for that here.  */
  if (*s == '(')
    {
      ++s;
      if (!*s || *s != '%'
	  || (*s == '%' && !isalpha (s[1])))
	error (_("Invalid register name on expression `%s'."),
	       eval_info->saved_expr);
      ++s;
      /* The presence of parenthesis means that we want to indirect
	 the register.  */
      indirect_p = 1;
    }
  else if (*s == '%')
    {
      ++s;
      if (!*s || !isalpha (*s))
	error (_("Invalid register name on expression `%s'."),
	       eval_info->saved_expr);
    }
  else
    error (_("Invalid register name on expression `%s'."),
	   eval_info->saved_expr);

  if (displacement && !indirect_p)
    /* We cannot apply displacement to non-indirect register access.  */
    error (_("Trying to apply displacement without indirecting register "
	     "on expression `%s'."), eval_info->saved_expr);

  /* Ok, let's calculate the size of the register name.  */
  start = s;
  while (isalnum (*s))
    ++s;

  len = s - start;

  if (indirect_p && *s == ')')
    ++s;

  regname = alloca (len + 1);
  strncpy (regname, start, len);
  regname[len] = '\0';

  /* Translating the register name into the corresponding number.  */
  regnum = user_reg_map_name_to_regnum (gdbarch, regname, len);

  if (regnum == -1)
    error (_("Invalid register name `%s' on expression `%s'."),
	   regname, eval_info->saved_expr);

  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
    ax_reg (eval_info->aexpr, regnum);
  else
    ret = value_of_register (regnum, frame);

  if (indirect_p)
    {
      struct type *t = NULL;
      enum agent_op aop = aop_ref32;

      /* If the user has specified that the register must be indirected,
	 we should know what's the correct type to cast it before making
	 the indirection.  This type corresponds to the bitness specified
	 before the `@' sign on the argument string, or it defaults to
	 `unsigned long' if the `@' were not present.  */

      switch (bitness)
	{
	case STAP_ARG_BITNESS_UNDEFINED:
	  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
	    {
	      if (gdbarch_addr_bit (gdbarch) == 32)
		aop = aop_ref32;
	      else
		aop = aop_ref64;
	    }
	  else
	    {
	      if (gdbarch_addr_bit (gdbarch) == 32)
		t = lookup_pointer_type
		  (builtin_type (gdbarch)->builtin_uint32);
	      else
		t = lookup_pointer_type
		  (builtin_type (gdbarch)->builtin_uint64);
	    }
	  break;

	case STAP_ARG_BITNESS_32BIT_SIGNED:
	  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
	    aop = aop_ref32;
	  else
	    t = lookup_pointer_type
	      (builtin_type (gdbarch)->builtin_int32);
	  break;

	case STAP_ARG_BITNESS_32BIT_UNSIGNED:
	  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
	    aop = aop_ref32;
	  else
	    t = lookup_pointer_type
	      (builtin_type (gdbarch)->builtin_uint32);
	  break;

	case STAP_ARG_BITNESS_64BIT_SIGNED:
	  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
	    aop = aop_ref64;
	  else
	    t = lookup_pointer_type
	      (builtin_type (gdbarch)->builtin_int64);
	  break;

	case STAP_ARG_BITNESS_64BIT_UNSIGNED:
	  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
	    aop = aop_ref64;
	  else
	    t = lookup_pointer_type
	      (builtin_type (gdbarch)->builtin_uint64);
	  break;

	default:
	  internal_error (__FILE__, __LINE__,
			  _("Undefined bitness for probe."));
	  break;
	}

      if (displacement)
	{
	  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
	    {
	      ax_const_l (eval_info->aexpr, value_as_long (displacement));
	      ax_simple (eval_info->aexpr, aop_add);
	    }
	  else
	    ret = value_ptradd (ret, value_as_long (displacement));
	}

      if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
	{
	  if (trace_kludge)
	    {
	      gdb_assert (aop == aop_ref32 || aop == aop_ref64);
	      ax_trace_quick (eval_info->aexpr, aop == aop_ref32 ? 4 : 8);
	    }
	  ax_simple (eval_info->aexpr, aop);
	}
      else
	{
	  ret = value_cast (t, ret);
	  ret = value_ind (ret);
	}
    }

  /* Updating the expression buffer pointer, because we have made
     some modifications to it before.  */
  eval_info->exp_buf = s;

  return ret;
}

/* This function tries to evaluate a single operand of the expression.

   Single operands can be:

   - unary operators `-' and `~';
   - integer constants (beginning with `$');
   - register access, with/out displacement and indirection.  */

static struct value *
stap_evaluate_single_operand (struct stap_evaluation_info *eval_info)
{
  struct gdbarch *gdbarch = eval_info->gdbarch;
  struct frame_info *frame = eval_info->frame;
  enum stap_arg_bitness bitness = eval_info->bitness;
  struct value *res = NULL;

  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
    /* If we are compiling, we cannot return NULL because that would
       lead to errors in future evaluations.  That's why we just make
       this dummy value, representing that the return value of this
       function is not NULL.  */
    res = value_from_longest (builtin_type (gdbarch)->builtin_int, 0);

  switch (*eval_info->exp_buf)
    {
    case '-': case '~':
	{
	  char c = *eval_info->exp_buf;

	  /* This is an unary operator (either `-' or `~').

	     If it is followed by a parenthesis, and this parenthesis
	     is NOT followed by a `%', then we are dealing with an expression
	     like `-(2 + 3)' or `~(2 + 3)'.  We just have to treat separately
	     and return the result after applying the operation (`-' or `~').

	     If it is followed by a digit, then we have only one choice:  it
	     is a displacement argument for a register access, like
	     `-4(%eax)'.  It also means that the operator can *only* be `-',
	     and the characters immediately after the number *must* be `(%'.

	     If it is followed by a `$', then it is an integer constant, and
	     we should apply the correct operation to it.  */

	  ++eval_info->exp_buf;
	  eval_info->exp_buf = skip_spaces (eval_info->exp_buf);
	  if (*eval_info->exp_buf
	      && *eval_info->exp_buf == '('
	      && eval_info->exp_buf[1] != '%')
	    {
	      struct value *tmp_res;

	      /* We're not dealing with a register name, but with an
		 expression like `-(2 + 3)' or `~(2 + 3)'.  We first have
		 to evaluate the right side of the expression (i.e., the
		 parenthesis), and then apply the specified operation
		 (either `-' or `~') to it.  */
	      tmp_res = stap_evaluate_conditionally (eval_info);

	      if (c == '-')
		{
		  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
		    {
		      /* We have to add `-1' to the stack, and multiply
			 the two values.  */
		      ax_const_l (eval_info->aexpr, -1);
		      ax_simple (eval_info->aexpr, aop_mul);
		    }
		  else
		    res = value_neg (tmp_res);
		}
	      else
		{
		  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
		    ax_simple (eval_info->aexpr, aop_bit_not);
		  else
		    res = value_complement (tmp_res);
		}
	    }
	  else if (isdigit (*eval_info->exp_buf))
	    {
	      int number;

	      /* This is a number, so it MUST be a register displacement.
		 The only operator allowed here is `-', it MUST be
		 followed by a number, and the number MUST be followed by
		 `(%'.  */
	      if (c != '-')
		error (_("Invalid operator `%c' for register displacement "
			 "on expression `%s'."), c, eval_info->saved_expr);

	      number = strtol (eval_info->exp_buf,
			       &eval_info->exp_buf, 0) * -1;

	      if (!*eval_info->exp_buf
		  || *eval_info->exp_buf != '('
		  || (*eval_info->exp_buf == '('
		      && eval_info->exp_buf[1] != '%'))
		error (_("Invalid method of indirecting a register on "
			 "expression `%s'."), eval_info->saved_expr);

	      res
		= value_from_longest (builtin_type (gdbarch)->builtin_int,
				      number);

	      res = stap_fetch_reg_value (eval_info, res);
	    }
	  else if (*eval_info->exp_buf == '$')
	    {
	      int number;

	      /* Last case.  We are dealing with an integer constant, so
		 we must read it and then apply the necessary operation,
		 either `-' or `~'.  */
	      ++eval_info->exp_buf;
	      number = strtol (eval_info->exp_buf,
			       &eval_info->exp_buf, 0);

	      if (!STAP_COMPILING_AGENT_EXPR_P (eval_info))
		res
		  = value_from_longest (builtin_type (gdbarch)->builtin_int,
					number);
	      else
		ax_const_l (eval_info->aexpr, number);

	      eval_info->exp_buf = skip_spaces (eval_info->exp_buf);

	      if (c == '-')
		{
		  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
		    ax_simple (eval_info->aexpr, aop_log_not);
		  else
		    res = value_neg (res);
		}
	      else
		{
		  if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
		    ax_simple (eval_info->aexpr, aop_bit_not);
		  else
		    res = value_complement (res);
		}
	    }
	  else
	    error (_("Invalid operand to unary operator `%c' on "
		     "expression `%s'."), c, eval_info->saved_expr);
	}
      break;

    case '0': case '1': case '2': case '3': case '4':
    case '5': case '6': case '7': case '8': case '9':
      {
	int number = strtol (eval_info->exp_buf, &eval_info->exp_buf, 0);

	/* This is a register displacement with a positive value.  We read
	   the number, and then check for the mandatory `(%' part.  */
	if (!*eval_info->exp_buf
	    || !(*eval_info->exp_buf == '('
		 && eval_info->exp_buf[1] == '%'))
	  error (_("Invalid register access on expression `%s'."),
		 eval_info->saved_expr);

	res = value_from_longest (builtin_type (gdbarch)->builtin_int,
				  number);

	res = stap_fetch_reg_value (eval_info, res);
      }
    break;

    case '$':
      {
	int number;

	/* This is an integer constant.  We just have to read the number
	   and return it.  */
	++eval_info->exp_buf;
	eval_info->exp_buf = skip_spaces (eval_info->exp_buf);

	number = strtol (eval_info->exp_buf, &eval_info->exp_buf, 0);

	if (STAP_COMPILING_AGENT_EXPR_P (eval_info))
	  ax_const_l (eval_info->aexpr, number);
	else
	  res = value_from_longest (builtin_type (gdbarch)->builtin_int,
				    number);

	eval_info->exp_buf = skip_spaces (eval_info->exp_buf);
      }
    break;

    case '(': case '%':
      {
	/* Register access, with or without indirection.  */
	res = stap_fetch_reg_value (eval_info, /*displacement=*/NULL);
	eval_info->exp_buf = skip_spaces (eval_info->exp_buf);
      }
    break;

    default:
      {
	error (_("Operator `%c' not recognized on expression `%s'."),
	       *eval_info->exp_buf, eval_info->saved_expr);
      }
    }

  return res;
}

/* This function is responsible for checking the necessary type of evaluation
   depending on what is the next "thing" in the buffer.  Valid values are:

   - Unary operators;
   - Integer constants;
   - Register displacement, indirection, and direct access;
   - Parenthesized operand.  */

static struct value *
stap_evaluate_conditionally (struct stap_evaluation_info *eval_info)
{
  char *s = eval_info->exp_buf;
  struct value *ret = NULL;

  if (*s == '-' || *s == '~' /* Unary operators.  */
      || *s == '$' /* Number (integer constant).  */
      || (isdigit (*s) && s[1] == '(' && s[2] == '%') /* Displacement.  */
      || (*s == '(' && s[1] == '%') /* Register indirection.  */
      || (*s == '%' && isalpha (s[1]))) /* Register value.  */
    /* This is a single operand, so just evaluate it and return.  */
    ret = stap_evaluate_single_operand (eval_info);
  else if (*s == '(')
    {
      /* We are dealing with a parenthesized operand.  It means we
	 have to evaluate it as it was a separate expression, without
	 left-side or precedence.  */
      ++eval_info->exp_buf;
      eval_info->exp_buf = skip_spaces (eval_info->exp_buf);

      ret = stap_evaluate_probe_argument_2 (eval_info,
					    /*lhs=*/NULL, /*prec=*/0);

      if (*eval_info->exp_buf != ')')
	error (_("Missign close-paren on expression `%s'."),
	       eval_info->saved_expr);

      ++eval_info->exp_buf;
      eval_info->exp_buf = skip_spaces (eval_info->exp_buf);
    }
  else
    error (_("Cannot evaluate expression `%s'."),
	   eval_info->saved_expr);

  return ret;
}

/* Evaluation function for probe's argument expressions.  LHS represents
   the left side of the expression, and PREC is the precedence of the
   last operator identified before calling the function.  */

static struct value *
stap_evaluate_probe_argument_2 (struct stap_evaluation_info *eval_info,
				struct value *lhs, int prec)
{
  struct value *rhs = NULL;
  int compiling_p = STAP_COMPILING_AGENT_EXPR_P (eval_info);

  /* This is an operator-precedence parser and evaluator.

     We work with left- and right-sides of expressions, and
     evaluate them depending on the precedence of the operators
     we find.  */

  eval_info->exp_buf = skip_spaces (eval_info->exp_buf);

  if (!lhs)
    /* We were called without a left-side, either because this is the
       first call, or because we were called to evaluate a parenthesized
       expression.  It doesn't really matter; we have to evaluate the
       left-side in order to continue the process.  */
    lhs = stap_evaluate_conditionally (eval_info);

  /* Start to evaluate the right-side, and to "join" left and right sides
     depending on the operation specified.

     This loop shall continue until we run out of characters in the input,
     or until we find a close-parenthesis, which means that we've reached
     the end of a sub-expression.  */
  while (eval_info->exp_buf
	 && *eval_info->exp_buf
	 && *eval_info->exp_buf != ')')
    {
      char *tmp_exp_buf;
      enum exp_opcode opcode;
      int cur_prec;

      if (!stap_is_operator (eval_info->exp_buf))
	error (_("Invalid operator `%c' on expression `%s'."),
	       *eval_info->exp_buf, eval_info->saved_expr);

      /* We have to save the current value of the expression buffer because
	 the `stap_get_opcode' modifies it in order to get the current
	 operator.  If this operator's precedence is lower than PREC, we
	 should return and not advance the expression buffer pointer.  */
      tmp_exp_buf = eval_info->exp_buf;
      stap_get_opcode (&tmp_exp_buf, &opcode);

      cur_prec = stap_get_operator_prec (opcode);
      if (cur_prec < prec)
	/* If the precedence of the operator that we are seeing now is
	   lower than the precedence of the first operator seen before
	   this evaluation process began, it means we should stop evaluating
	   and return.  */
	break;

      eval_info->exp_buf = tmp_exp_buf;
      eval_info->exp_buf = skip_spaces (eval_info->exp_buf);

      /* Evaluate the right-side of the expression.  */
      rhs = stap_evaluate_conditionally (eval_info);

      /* While we still have operators, try to evaluate another
	 right-side, but using the current right-side as a left-side.  */
      while (*eval_info->exp_buf
	     && stap_is_operator (eval_info->exp_buf))
	{
	  enum exp_opcode lookahead_opcode;
	  int lookahead_prec;

	  /* Saving the current expression buffer position.  The explanation
	     is the same as above.  */
	  tmp_exp_buf = eval_info->exp_buf;
	  stap_get_opcode (&tmp_exp_buf, &lookahead_opcode);
	  lookahead_prec = stap_get_operator_prec (lookahead_opcode);

	  if (lookahead_prec <= prec)
	    /* If we are dealing with an operator whose precedence is lower
	       than the first one, just abandon the attempt.  */
	    break;

	  rhs = stap_evaluate_probe_argument_2 (eval_info,
						rhs, lookahead_prec);
	}

      /* Now, "join" both left and right sides into one left-side, using
	 the specified operator.  */
      if (compiling_p)
	stap_opcode_to_ax (eval_info, opcode);
      else
	lhs = value_binop (lhs, rhs, opcode);
    }

  return lhs;
}

/* This function fills the necessary arguments for the evaluation function
   to work.  */

static struct value *
stap_evaluate_probe_argument_1 (struct objfile *objfile,
				const struct stap_probe *probe,
				struct frame_info *frame,
				int n)
{
  struct stap_evaluation_info eval_info;
  char *s = (char *) probe->parsed_args->arg[n].arg_str;
  struct value *res, *vs[4];

  /* Filling necessary information for evaluation function.  */
  eval_info.saved_expr = s;
  eval_info.exp_buf = s;
  eval_info.gdbarch = get_objfile_arch (objfile);
  eval_info.frame = frame;
  eval_info.bitness = probe->parsed_args->arg[n].bitness;
  /* We are not compiling to an agent expression.  */
  eval_info.aexpr = NULL;
  eval_info.avalue = NULL;

  res = stap_evaluate_probe_argument_2 (&eval_info,
					/*lhs=*/NULL, /*prec=*/0);

  if (!res)
    error (_("Could not evaluate expression `%s'."),
	   eval_info.saved_expr);

  return res;
}

/* See definition in stap-probe.h.  */

struct value *
stap_evaluate_probe_argument (struct objfile *objfile,
			      const struct stap_probe *probe,
			      struct frame_info *frame,
			      int n)
{
  if (!probe->parsed_args)
    stap_parse_probe_arguments ((struct stap_probe *) probe);

  if (!probe->parsed_args->arg
      || n >= probe->parsed_args->n_args)
    return NULL;

  return stap_evaluate_probe_argument_1 (objfile, probe, frame, n);
}

/* Helper function which compiles the probe's argument N into an
   agent expression, suitable for using with tracepoints.  */

static void
stap_compile_to_ax_1 (struct objfile *objfile,
		      const struct stap_probe *probe,
		      struct agent_expr *expr,
		      struct axs_value *value,
		      int n)
{
  struct stap_evaluation_info eval_info;
  struct gdbarch *gdbarch = expr->gdbarch;
  char *s = (char *) probe->parsed_args->arg[n].arg_str;

  /* Filling necessary information for evaluation function.  */
  eval_info.saved_expr = s;
  eval_info.exp_buf = s;
  eval_info.gdbarch = expr->gdbarch;
  eval_info.frame = NULL;
  eval_info.bitness = probe->parsed_args->arg[n].bitness;
  /* We are compiling to an agent expression.  */
  eval_info.aexpr = expr;
  eval_info.avalue = value;

  /* We can always use this kind.  */
  value->kind = axs_rvalue;

  /* Figuring out the correct type for this axs_value.  */
  switch (eval_info.bitness)
    {
    case STAP_ARG_BITNESS_UNDEFINED:
      if (gdbarch_addr_bit (gdbarch) == 32)
	value->type = builtin_type (gdbarch)->builtin_uint32;
      else
	value->type = builtin_type (gdbarch)->builtin_uint64;
      break;

    case STAP_ARG_BITNESS_32BIT_SIGNED:
      value->type = builtin_type (gdbarch)->builtin_int32;
      break;

    case STAP_ARG_BITNESS_32BIT_UNSIGNED:
      value->type = builtin_type (gdbarch)->builtin_uint32;
      break;

    case STAP_ARG_BITNESS_64BIT_SIGNED:
      value->type = builtin_type (gdbarch)->builtin_int64;
      break;

    case STAP_ARG_BITNESS_64BIT_UNSIGNED:
      value->type = builtin_type (gdbarch)->builtin_uint64;
      break;

    default:
      internal_error (__FILE__, __LINE__,
		      _("Undefined bitness for probe."));
      break;
    }

  stap_evaluate_probe_argument_2 (&eval_info,
				  /*lhs=*/NULL, /*prec=*/0);
}

/* See definition in stap-probe.h.  */

void
stap_compile_to_ax (struct objfile *objfile,
		    const struct stap_probe *probe,
		    struct agent_expr *expr,
		    struct axs_value *value,
		    int n)
{
  if (!probe->parsed_args)
    stap_parse_probe_arguments ((struct stap_probe *) probe);

  if (!probe->parsed_args->arg
      || n >= probe->parsed_args->n_args)
    return;

  stap_compile_to_ax_1 (objfile, probe, expr, value, n);
}

struct value *
stap_safe_evaluate_at_pc (struct frame_info *frame, int n)
{
  const struct stap_probe *probe;
  struct objfile *objfile;
  int n_probes;

  probe = find_probe_by_pc (get_frame_pc (frame), &objfile);
  if (!probe)
    return NULL;
  gdb_assert (objfile->sf && objfile->sf->sym_probe_fns);

  n_probes
    = objfile->sf->sym_probe_fns->sym_get_probe_argument_count (objfile,
								probe);
  if (n >= n_probes)
    return NULL;

  return objfile->sf->sym_probe_fns->sym_evaluate_probe_argument (objfile,
								  probe,
								  frame,
								  n);
}

/* This function frees the space allocated to hold information about
   the probe's parsed arguments.  */

void
stap_free_parsed_args (struct stap_args_info *parsed_args)
{
  int i;

  if (!parsed_args
      || parsed_args == &dummy_stap_args_info
      || parsed_args->n_args == 0)
    return;

  for (i = 0; i < parsed_args->n_args; i++)
    xfree (parsed_args->arg);

  xfree (parsed_args);
}

/* A utility structure.  A VEC of these is built when handling "info
   probes".  */

struct stap_probe_and_objfile
{
  /* The probe.  */
  const struct stap_probe *probe;
  /* The probe's objfile.  */
  struct objfile *objfile;
};

typedef struct stap_probe_and_objfile stap_entry;
DEF_VEC_O (stap_entry);

/* A helper function for collect_probes that compiles a regexp and
   throws an exception on error.  This installs a cleanup to free the
   resulting pattern on success.  If RX is NULL, this does nothing.  */

static void
compile_rx_or_error (regex_t *pattern, const char *rx, const char *message)
{
  int code;

  if (!rx)
    return;

  code = regcomp (pattern, rx, REG_NOSUB);
  if (code == 0)
    make_regfree_cleanup (pattern);
  else
    {
      char *err = get_regcomp_error (code, pattern);

      make_cleanup (xfree, err);
      error (_("%s: %s"), message, err);
    }
}

/* Make a vector of probes matching OBJNAME, PROVIDER, and PROBE.
   Each argument is a regexp, or NULL, which matches anything.  */

static VEC (stap_entry) *
collect_probes (char *objname, char *provider, char *probe)
{
  struct objfile *objfile;
  VEC (stap_entry) *result = NULL;
  struct cleanup *cleanup;
  regex_t obj_pat, prov_pat, probe_pat;

  cleanup = make_cleanup (VEC_cleanup (stap_entry), &result);

  compile_rx_or_error (&prov_pat, provider, _("Invalid provider regexp"));
  compile_rx_or_error (&probe_pat, probe, _("Invalid probe regexp"));
  compile_rx_or_error (&obj_pat, objname, _("Invalid object file regexp"));

  ALL_OBJFILES (objfile)
  {
    const struct stap_probe *probes;
    int i, num_probes;

    if (! objfile->sf || ! objfile->sf->sym_probe_fns)
      continue;

    if (objname)
      {
	if (regexec (&obj_pat, objfile->name, 0, NULL, 0) != 0)
	  continue;
      }

    probes = objfile->sf->sym_probe_fns->sym_get_probes (objfile, &num_probes);
    for (i = 0; i < num_probes; ++i)
      {
	stap_entry entry;

	if (provider)
	  {
	    if (regexec (&prov_pat, probes[i].provider, 0, NULL, 0) != 0)
	      continue;
	  }

	if (probe)
	  {
	    if (regexec (&probe_pat, probes[i].name, 0, NULL, 0) != 0)
	      continue;
	  }

	entry.probe = &probes[i];
	entry.objfile = objfile;
	VEC_safe_push (stap_entry, result, &entry);
      }
  }

  discard_cleanups (cleanup);
  return result;
}

/* A qsort comparison function for stap_entry objects.  */

static int
compare_entries (const void *a, const void *b)
{
  const stap_entry *ea = a;
  const stap_entry *eb = b;
  int v;

  v = strcmp (ea->probe->provider, eb->probe->provider);
  if (v)
    return v;

  v = strcmp (ea->probe->name, eb->probe->name);
  if (v)
    return v;

  if (ea->probe->address < eb->probe->address)
    return -1;
  if (ea->probe->address > eb->probe->address)
    return 1;

  return strcmp (ea->objfile->name, eb->objfile->name);
}

/* Implementation of the "info probes" command.  */

static void
info_probes_command (char *arg, int from_tty)
{
  char *provider, *probe = NULL, *objname = NULL;
  struct cleanup *cleanup = make_cleanup (null_cleanup, NULL);
  VEC (stap_entry) *items;
  int i, addr_width, any_found;
  stap_entry *entry;

  provider = extract_arg (&arg);
  if (provider)
    {
      make_cleanup (xfree, provider);

      probe = extract_arg (&arg);
      if (probe)
	{
	  make_cleanup (xfree, probe);

	  objname = extract_arg (&arg);
	  if (objname)
	    make_cleanup (xfree, objname);
	}
    }

  items = collect_probes (objname, provider, probe);
  make_cleanup (VEC_cleanup (stap_entry), &items);
  make_cleanup_ui_out_table_begin_end (uiout, 5,
				       VEC_length (stap_entry, items),
				       "SystemTapProbes");

  if (! VEC_empty (stap_entry, items))
    qsort (VEC_address (stap_entry, items),
	   VEC_length (stap_entry, items),
	   sizeof (stap_entry),
	   compare_entries);

  addr_width = 4 + (gdbarch_ptr_bit (get_current_arch ()) / 4);

  ui_out_table_header (uiout, 10, ui_left, "provider", _("Provider"));
  ui_out_table_header (uiout, 10, ui_left, "name", _("Name"));
  ui_out_table_header (uiout, addr_width - 1, ui_left, "addr", _("Where"));
  ui_out_table_header (uiout, addr_width - 1, ui_left, "semaphore",
		       _("Semaphore"));
  ui_out_table_header (uiout, 30, ui_left, "object", _("Object"));
  ui_out_table_body (uiout);

  for (i = 0; VEC_iterate (stap_entry, items, i, entry); ++i)
    {
      struct cleanup *inner;

      inner = make_cleanup_ui_out_tuple_begin_end (uiout, "probe");

      ui_out_field_string (uiout, "provider", entry->probe->provider);
      ui_out_field_string (uiout, "name", entry->probe->name);
      ui_out_field_core_addr (uiout, "addr", get_current_arch (),
			      entry->probe->address);
      if (entry->probe->sem_addr == 0)
	ui_out_field_skip (uiout, "semaphore");
      else
      ui_out_field_core_addr (uiout, "semaphore", get_current_arch (),
			      entry->probe->sem_addr);
      ui_out_field_string (uiout, "object", entry->objfile->name);
      ui_out_text (uiout, "\n");

      do_cleanups (inner);
    }

  any_found = ! VEC_empty (stap_entry, items);
  do_cleanups (cleanup);

  if (! any_found)
    ui_out_message (uiout, 0, _("No probes matched.\n"));
}



/* See definition in stap-probe.h.  */

const struct stap_probe *
find_probe_in_objfile (struct objfile *objfile,
		       const char *provider,
		       const char *name)
{
  const struct stap_probe *probes;
  int i, num_probes;

  if (! objfile->sf || ! objfile->sf->sym_probe_fns)
    return NULL;

  probes = objfile->sf->sym_probe_fns->sym_get_probes (objfile, &num_probes);
  for (i = 0; i < num_probes; ++i)
    {
      if (strcmp (probes[i].provider, provider) != 0)
	continue;

      if (strcmp (probes[i].name, name) != 0)
	continue;

      return &probes[i];
    }

  return NULL;
}

/* See definition in stap-probe.h.  */

struct symtabs_and_lines
parse_stap_probe (char **argptr, struct linespec_result *canonical)
{
  char *full_arg = extract_arg (argptr);
  char *arg = xstrdup (full_arg);
  char *objfile_name = NULL, *provider = NULL, *name, *p;
  struct cleanup *cleanup;
  struct symtabs_and_lines result;
  struct objfile *objfile;

  result.sals = NULL;
  result.nelts = 0;

  /* The caller ensured that this starts with 'probe:'.  */
  gdb_assert (arg && strncmp (arg, "probe:", 6) == 0);
  cleanup = make_cleanup (xfree, arg);
  make_cleanup (xfree, full_arg);
  arg += 6;

  /* Extract each word from the argument, separated by ":"s.  */
  p = strchr (arg, ':');
  if (p == NULL)
    {
      /* This is `probe:name'.  */
      name = arg;
    }
  else
    {
      char *hold = p + 1;

      *p = '\0';
      p = strchr (hold, ':');
      if (p == NULL)
	{
	  /* This is `probe:provider:name'.  */
	  provider = arg;
	  name = hold;
	}
      else
	{
	  /* This is `probe:objfile:provider:name'.  */
	  *p = '\0';
	  objfile_name = arg;
	  provider = hold;
	  name = p + 1;
	}
    }

  if (*name == '\0')
    error (_("no probe name specified"));
  if (provider && *provider == '\0')
    error (_("invalid provider name"));
  if (objfile_name && *objfile_name == '\0')
    error (_("invalid objfile name"));

  if (canonical)
    canonical->canonical = NULL;

  ALL_OBJFILES (objfile)
  {
    const struct stap_probe *probes;
    int i, num_probes;

    if (! objfile->sf || ! objfile->sf->sym_probe_fns)
      continue;

    if (objfile_name
	&& FILENAME_CMP (objfile->name, objfile_name) != 0
	&& FILENAME_CMP (lbasename (objfile->name), objfile_name) != 0)
      continue;

    probes = objfile->sf->sym_probe_fns->sym_get_probes (objfile, &num_probes);
    for (i = 0; i < num_probes; ++i)
      {
	struct symtab_and_line *sal;

	if (provider && strcmp (probes[i].provider, provider) != 0)
	  continue;

	if (strcmp (probes[i].name, name) != 0)
	  continue;

	++result.nelts;
	result.sals = xrealloc (result.sals,
				result.nelts * sizeof (struct symtab_and_line));
	sal = &result.sals[result.nelts - 1];

	init_sal (sal);

	sal->pc = probes[i].address;
	sal->explicit_pc = 1;
	sal->section = find_pc_overlay (sal->pc);
	sal->pspace = current_program_space;
	sal->semaphore = probes[i].sem_addr;

	if (canonical)
	  {
	    canonical->canonical = xrealloc (canonical->canonical,
					     result.nelts * sizeof (char **));
	    canonical->canonical[result.nelts - 1] = xstrdup (full_arg);
	  }
      }
  }

  if (result.nelts == 0)
    {
      throw_error (NOT_FOUND_ERROR,
		   _("No probe matching objfile=`%s', provider=`%s', name=`%s'"),
		   objfile_name ? objfile_name : _("<any>"),
		   provider ? provider : _("<any>"),
		   name);
    }

  if (canonical)
    {
      canonical->special_display = 1;
      canonical->pre_expanded = 1;
    }

  do_cleanups (cleanup);

  return result;
}



/* See definition in stap-probe.h.  */

const struct stap_probe *
find_probe_by_pc (CORE_ADDR pc, struct objfile **objfile_out)
{
  struct objfile *objfile;

  ALL_OBJFILES (objfile)
  {
    const struct stap_probe *probes;
    int i, num_probes;
    stap_entry entry;

    if (! objfile->sf || ! objfile->sf->sym_probe_fns)
      continue;

    /* If this proves too inefficient, we can replace with a hash.  */
    probes = objfile->sf->sym_probe_fns->sym_get_probes (objfile, &num_probes);
    for (i = 0; i < num_probes; ++i)
      {
	if (probes[i].address == pc)
	  {
	    *objfile_out = objfile;
	    return &probes[i];
	  }
      }
  }

  return NULL;
}

/* This is called to compute the value of one of the $_probe_arg*
   convenience variables.  */

static struct value *
compute_probe_arg (struct gdbarch *arch, struct internalvar *ivar,
		   void *data)
{
  struct frame_info *frame = get_selected_frame (_("No frame selected"));
  CORE_ADDR pc = get_frame_pc (frame);
  int sel = (int) (uintptr_t) data;
  struct objfile *objfile;
  const struct stap_probe *pc_probe;
  int n_probes;

  /* SEL==10 means "_probe_argc".  */
  gdb_assert (sel >= 0 && sel <= STAP_MAX_ARGS);

  pc_probe = find_probe_by_pc (pc, &objfile);
  if (pc_probe == NULL)
    error (_("No SystemTap probe at PC %s"), core_addr_to_string (pc));

  n_probes
    = objfile->sf->sym_probe_fns->sym_get_probe_argument_count (objfile,
								pc_probe);
  if (sel == 10)
    return value_from_longest (builtin_type (arch)->builtin_int, n_probes);

  if (sel >= n_probes)
    error (_("Invalid probe argument %d -- probe has %d arguments available"),
	   sel, n_probes);

  return objfile->sf->sym_probe_fns->sym_evaluate_probe_argument (objfile,
								  pc_probe,
								  frame, sel);
}

/* This is called to compile one of the $_probe_arg* convenience
   variables into an agent expression.  */

static void
compile_probe_arg (struct internalvar *ivar, struct agent_expr *expr,
		   struct axs_value *value, void *data)
{
  CORE_ADDR pc = expr->scope;
  int sel = (int) (uintptr_t) data;
  struct objfile *objfile;
  const struct stap_probe *pc_probe;
  int n_probes;

  /* SEL==10 means "_probe_argc".  */
  gdb_assert (sel >= 0 && sel <= 10);

  pc_probe = find_probe_by_pc (pc, &objfile);
  if (pc_probe == NULL)
    error (_("No SystemTap probe at PC %s"), core_addr_to_string (pc));

  n_probes
    = objfile->sf->sym_probe_fns->sym_get_probe_argument_count (objfile,
								pc_probe);
  if (sel == 10)
    {
      value->kind = axs_rvalue;
      value->type = builtin_type (expr->gdbarch)->builtin_int;
      ax_const_l (expr, n_probes);
      return;
    }

  gdb_assert (sel >= 0);
  if (sel >= n_probes)
    error (_("Invalid probe argument %d -- probe has %d arguments available"),
	   sel, n_probes);

  objfile->sf->sym_probe_fns->sym_compile_to_ax (objfile, pc_probe,
						 expr, value, sel);
}



/* Implementation of `$_probe_arg*' set of variables.  */

static const struct internalvar_funcs probe_funcs =
{
  compute_probe_arg,
  compile_probe_arg,
  NULL
};

void
_initialize_stap_probe (void)
{
  add_info ("probes", info_probes_command, _("\
Show available static probes.\n\
Usage: info probes [PROVIDER [NAME [OBJECT]]]\n\
Each argument is a regular expression, used to select probes.\n\
PROVIDER matches probe provider names.\n\
NAME matches the probe names.\n\
OBJECT match the executable or shared library name."));

  create_internalvar_type_lazy ("_probe_argc", &probe_funcs,
				(void *) (uintptr_t) 10);
  create_internalvar_type_lazy ("_probe_arg0", &probe_funcs,
				(void *) (uintptr_t) 0);
  create_internalvar_type_lazy ("_probe_arg1", &probe_funcs,
				(void *) (uintptr_t) 1);
  create_internalvar_type_lazy ("_probe_arg2", &probe_funcs,
				(void *) (uintptr_t) 2);
  create_internalvar_type_lazy ("_probe_arg3", &probe_funcs,
				(void *) (uintptr_t) 3);
  create_internalvar_type_lazy ("_probe_arg4", &probe_funcs,
				(void *) (uintptr_t) 4);
  create_internalvar_type_lazy ("_probe_arg5", &probe_funcs,
				(void *) (uintptr_t) 5);
  create_internalvar_type_lazy ("_probe_arg6", &probe_funcs,
				(void *) (uintptr_t) 6);
  create_internalvar_type_lazy ("_probe_arg7", &probe_funcs,
				(void *) (uintptr_t) 7);
  create_internalvar_type_lazy ("_probe_arg8", &probe_funcs,
				(void *) (uintptr_t) 8);
  create_internalvar_type_lazy ("_probe_arg9", &probe_funcs,
				(void *) (uintptr_t) 9);
}
